/**
 * Objet de gestion de cache
 * @param {String} nom - nom unique du cache. Sert egalement de cle dans le stockage local
 * @param {CacheFactory} factory - factory de gestion de cache qui doit �tendre CacheFactory
 * @param {int} delai - delai de rafraichissement. Si <= 0 le cache ne sera rafraichit qu'au demarrage
 * 
 * @exemple
var SampleCacheFactory = CacheFactory.extend({
	constructor: function() {
	},
	refresh : function(cache) {
		cache = cache || {};
		return FwMC.Core.doAjaxAsync("http://www.cache.com/cache/cache1.txt").then(function(res) {
			cache["test1"] = res.data;
			return FwMC.Core.doAjaxAsync("http://www.cache.com/cache/cache2.txt");
		}).then(function(res) {
			cache["test2"] = res.data;
			return cache;
		});
	},
	
	getBean: function(id, leCache) {
		return leCache[id];
	}
});
var monCache = new Cache("sampleCache", new SampleCacheFactory(), 0);
// Recuperation de la valeur en asynchrone
monCache.getBean("test1").then(function(res) {
}
// Recuperation en synchrone, si le cache n'est pas disponible ca renverra null
monCache.getBeanSync("test1");
 */
var Cache = FwMC.Class.extend((function() {
	var delai = 0,
		nom = null,
		factory = null,
		cache = {},
		promiseRefresh = null,
		exports = {};
	/**
	 * Construteur
	 * Lance un premier refresh en asynchrone
	 */
	exports.constructor = function(pNom, pFactory, pDelai) {
		nom = pNom;
		factory = pFactory;
		// Le delai n'est modifie que si renseigne
		if (pDelai) {
			delai = pDelai;
		}
		// Si le cache est deja en localstorage on le renvoie de suite sans creer de promesse
		// On fait un refresh mais le cache sera quand meme dispo
		if (window.localStorage[nom]) {
			cache = FwMC.Core.Json.parse(window.localStorage[nom]);
			refresh();
		} else {
			// Sinon on lance un refresh et on stocke la promesse pour pouvoir la reutiliser plus tard
			promiseRefresh = refresh().then(function() {
				promiseRefresh = null;
			});
		}
	};
	/**
	 * Recuperation d'un element du cache en asynchrone. Appelle la factory
	 * @param {String} id - identifiant de l'element recherche
	 * @return {Promise<Object>} promesse du resultat
	 */
	exports.getBean = function(id) {
		// S'il y a une promesse en cours on la renvoie
		if (promiseRefresh) {
			return promiseRefresh.then(function() {
				return Q(factory.getBean(id, cache));
			});
		} else {
			return Q(factory.getBean(id, cache));
		}
	};
	/**
	 * Recuperation des id du cache
	 * @return {Promise<Object>} promesse du resultat
	 */
	exports.getBeanNames = function() {
		// S'il y a une promesse en cours on la renvoie
		if (promiseRefresh) {
			return promiseRefresh.then(function() {
				return Q(factory.getBeanNames(cache));
			});
		} else {
			return Q(factory.getBeanNames(cache));
		}
	};
	/**
	 * Recuperation d'un element du cache en synchrone. Appelle la factory
	 * Si le cache n'est pas pret ca renverra forcement null.
	 * @param {String} id - identifiant de l'element recherche
	 * @return {Object} resultat
	 */
	exports.getBeanSync = function(id) {
		var result = null;
		if (cache) {
			result = factory.getBean(id, cache);
		}
		return result;
	};
	/**
	 * Recuperation des id du cache en synchrone. Appelle la factory
	 * Si le cache n'est pas pret ca renverra forcement null.
	 * @return {Object} resultat
	 */
	exports.getBeanNamesSync = function() {
		var result = null;
		if (cache) {
			result = factory.getBeanNames(cache);
		}
		return result;
	};
	/**
	 * Rafraichissement du cache
	 * Appelle la factory et ne modifie le cache qu'une fois le rafraichissement termine en recopie.
	 */
	function refresh() {
		return factory.refresh(cache ? $.extend({}, cache) : null).then(function(newCache) {
			if (newCache) {
				// On fait une copie pour eviter les pb de synchro
				cache = $.extend({}, newCache);
				window.localStorage[nom] = FwMC.Core.Json.stringify(cache);
			}
			if (delai > 0) {
				setTimeout(function() {refresh();}, delai);
			}
		}).fail(function(err) {
		});
	}
	return exports;
})());

/**
 * Factory de cache
 */
var CacheFactory = FwMC.Class.extend({
	cache: {
		value:null
	},
	contructor: function() {},
	/**
	 * Rafraichissement du cache. La methode doit renvoyer une promesse !
	 */
	refresh: function() {
		return Q({});
	},
	/**
	 * Recuperation d'un bean
	 * @param {String} id - identifiant recherche
	 * @param {Object} leCache - le cache dans l'etat actuel 
	 * @return {Object} element du cache
	 */
	getBean: function(id, leCache) {
		return null;
	}
});